@@ -212,14 +212,15 @@ class Page(object):
                 f = codecs.open(self._text_filename(), 'rb', config.charset)
             except IOError, er:
                 import errno
-                if er.errno == errno.ENOENT:
-                    # just doesn't exist, return empty text (note that we
-                    # never store empty pages, so this is detectable and also
-                    # safe when passed to a function expecting a string)
-                    return ""
+                if er.errno in [errno.ENOENT, errno.ENAMETOOLONG, ]:
+                    # ENOENT: doesn't exist, file not found
+                    # ENAMETOOLONG: can't exist, name too long for this fs
+                    # return empty text (note that we never store empty pages,
+                    # so this is detectable and also safe when passed to a
+                    # function expecting a string)
+                    return u""
                 else:
                     raise
-
             # read file content and make sure it is closed properly
             try:
                 text = f.read()
@@ -680,7 +681,7 @@ class Page(object):
             return os.path.getsize(self._text_filename(rev=rev))
         except EnvironmentError, e:
             import errno
-            if e.errno == errno.ENOENT:
+            if e.errno in [errno.ENOENT, errno.ENAMETOOLONG, ]:
                 return 0
             raise
 
@@ -811,8 +812,9 @@ class Page(object):
 
         return link
 
-    def getSubscribers(self, request, **kw):
+    def getSubscribersUncached(self, request, **kw):
         """ Get all subscribers of this page.
+            (original method, slow for many users, needs to open all user profiles)
 
         @param request: the request object
         @keyword include_self: if 1, include current user (default: 0)
@@ -834,6 +836,8 @@ class Page(object):
         else:
             from MoinMoin.security import Default as UserPerms
 
+        request.clock.start("getSubscribersUncached")
+
         # get email addresses of the all wiki user which have a profile stored;
         # add the address only if the user has subscribed to the page and
         # the user is not the current editor
@@ -872,6 +876,115 @@ class Page(object):
             else:
                 subscriber_list[lang].append(subscriber.email)
 
+        request.clock.stop("getSubscribersUncached")
+
+        return subscriber_list
+
+    def getSubscribers(self, request, **kw):
+        """ Get all subscribers of this page.
+            (new method, uses caching)
+
+        @param request: the request object
+        @keyword include_self: if 1, include current user (default: 0)
+        @keyword return_users: if 1, return user instances (default: 0)
+        @rtype: dict
+        @return: lists of subscribed email addresses in a dict by language key
+        """
+        include_self = kw.get('include_self', self.include_self)
+        return_users = kw.get('return_users', 0)
+
+        request.clock.start('getSubscribers')
+        # extract categories of this page
+        pageList = self.getCategories(request)
+
+        # add current page name for list matching
+        pageList.append(self.page_name)
+
+        scope, arena, key = 'userdir', 'users', 'pagesubscriptions'
+
+        # get or create cache file
+        cache = caching.CacheEntry(request, arena, key, scope=scope, use_pickle=True)
+        if cache.exists():
+            page_sub = cache.content()
+        else:
+            # build a cache if it doesn't exist
+            cache = caching.CacheEntry(request, arena, key, scope=scope, use_pickle=True, do_locking=False)
+            # lock to stop anybody else interfering with the data while we're working
+            cache.lock('w')
+            userlist = user.getUserList(request)
+            page_sub = {}
+            for uid in userlist:
+                subscriber = user.User(request, uid)
+                # we don't care about storing entries for users without any page subscriptions
+                if subscriber.subscribed_pages:
+                    page_sub[subscriber.id] = {
+                        'name': subscriber.name,
+                        'email': subscriber.email,
+                        'subscribed_pages': subscriber.subscribed_pages,
+                    }
+            cache.update(page_sub)
+            cache.unlock()
+
+        if self.cfg.SecurityPolicy:
+            UserPerms = self.cfg.SecurityPolicy
+        else:
+            from MoinMoin.security import Default as UserPerms
+
+        # get email addresses of all wiki users which have a profile stored;
+        # add the address only if the user has subscribed to the page and
+        # the user is not the current editor
+        userlist = page_sub
+        subscriber_list = {}
+
+        pages = pageList[:]
+        if request.cfg.interwikiname:
+            pages += ["%s:%s" % (request.cfg.interwikiname, pagename) for pagename in pageList]
+        # Create text for regular expression search
+        text = '\n'.join(pages)
+
+        for uid in userlist:
+            if uid == request.user.id and not include_self:
+                continue # no self notification
+
+            # This is a bit wrong if return_users=1 (which implies that the caller will process
+            # user attributes and may, for example choose to send an SMS)
+            # So it _should_ be "not (subscriber.email and return_users)" but that breaks at the moment.
+            if not userlist[uid]['email']:
+                continue # skip empty email addresses
+
+            # now check patterns for actual match
+            isSubscribed = False
+            for pattern in userlist[uid]['subscribed_pages']:
+                if pattern in pages:
+                    isSubscribed = True
+                    break
+                try:
+                    pattern = re.compile(r'^%s$' % pattern, re.M)
+                except re.error:
+                    continue
+                if pattern.search(text):
+                    isSubscribed = True
+                    break
+
+            # only if subscribed, create a User object from the profile
+            if isSubscribed:
+                subscriber = user.User(request, uid)
+
+                if not subscriber.valid:
+                    continue
+
+                if not UserPerms(subscriber).read(self.page_name):
+                    continue
+
+                lang = subscriber.language or request.cfg.language_default
+                if not lang in subscriber_list:
+                    subscriber_list[lang] = []
+                if return_users:
+                    subscriber_list[lang].append(subscriber)
+                else:
+                    subscriber_list[lang].append(subscriber.email)
+
+        request.clock.stop('getSubscribers')
         return subscriber_list
 
     def parse_processing_instructions(self):
@@ -1037,7 +1150,11 @@ class Page(object):
             redirect_url = Page(request, pagename).url(request,
                                                        querystr={'action': 'show', 'redirect': self.page_name, },
                                                        anchor=anchor)
-            request.http_redirect(redirect_url, code=301)
+            # we do NOT use 301 as a page edit may change the redirect to
+            # another target or even remove it again. so we can't really say
+            # it is permanent (301) or we might run into issues with clients
+            # like chrome that cache permanent redirects.
+            request.http_redirect(redirect_url, code=302)
             return
 
         # if necessary, load the formatter
