@@ -55,10 +55,10 @@ def _loadWordsFile(request, dict, filename):
     request.clock.start('spellread')
     try:
         try:
-            f = codecs.open(filename, 'rt', config.charset)
+            f = codecs.open(filename, 'r', config.charset)
             lines = f.readlines()
         except UnicodeError:
-            f = codecs.open(filename, 'rt', 'iso-8859-1')
+            f = codecs.open(filename, 'r', 'iso-8859-1')
             lines = f.readlines()
     finally:
         f.close()
@@ -102,7 +102,7 @@ def _addLocalWords(request):
     from MoinMoin.PageEditor import PageEditor
     # get the new words as a string (if any are marked at all)
     try:
-        newwords = request.form['newwords']
+        newwords = request.form.getlist('newwords')
     except KeyError:
         # no new words checked
         return
@@ -187,8 +187,8 @@ def checkSpelling(page, request, own_form=1):
 
         # add a form containing the bad words
         if own_form:
-            msg = msg + ('<form method="post" action="%s/%s">\n'
-                         '<input type="hidden" name="action" value="%s">\n') % (request.getScriptname(), wikiutil.quoteWikinameURL(page.page_name), action_name)
+            msg = msg + ('<form method="post" action="%s">\n'
+                         '<input type="hidden" name="action" value="%s">\n') % (request.href(page.page_name), action_name)
 
         checkbox = '<input type="checkbox" name="newwords" value="%(word)s">%(word)s&nbsp;&nbsp;'
         msg = msg + (
