@@ -10,25 +10,25 @@
 from MoinMoin import macro
 from MoinMoin.parser.text import Parser
 from MoinMoin.formatter.text_html import Formatter
-
+from MoinMoin._tests import become_trusted, create_page, make_macro, nuke_page
 
 class TestMacro:
+    pagename = u'AutoCreatedMoinMoinTemporaryTestPageForTestMacro'
+
+    def setup_class(self):
+        request = self.request
+        become_trusted(request)
+        self.page = create_page(request, self.pagename, u"Foo!")
+
+    def teardown_class(self):
+        nuke_page(self.request, self.pagename)
+
     def testTrivialMacro(self):
         """macro: trivial macro works"""
-        m = self._make_macro()
+        m = make_macro(self.request, self.page)
         expected = m.formatter.linebreak(0)
         result = m.execute("BR", "")
         assert result == expected
 
-    def _make_macro(self):
-        """Test helper"""
-        p = Parser("##\n", self.request)
-        p.formatter = Formatter(self.request)
-        self.request.formatter = p.formatter
-        p.form = self.request.form
-        m = macro.Macro(p)
-        return m
-
-
 coverage_modules = ['MoinMoin.macro']
 
