@@ -149,6 +149,9 @@ def sendmail(request, to, subject, text, mail_from=None):
                 except AttributeError:
                     # in case the connection failed, SMTP has no "sock" attribute
                     pass
+        except UnicodeError, e:
+            logging.exception("unicode error [%r -> %r]" % (mail_from, to, ))
+            return (0, str(e))
         except smtplib.SMTPException, e:
             logging.exception("smtp mail failed with an exception.")
             return (0, str(e))
