@@ -15,21 +15,34 @@
       * storage code
 
     @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
-                2003-2007 MoinMoin:ThomasWaldmann
+                2003-2013 MoinMoin:ThomasWaldmann,
+                2010 Michael Foetsch <foetsch@yahoo.com>
     @license: GNU GPL, see COPYING for details.
 """
 
-# add names here to hide them in the cgitb traceback
-unsafe_names = ("id", "key", "val", "user_data", "enc_password", "recoverpass_key")
-
 import os, time, codecs, base64
+from copy import deepcopy
+import md5crypt
+
+try:
+    import crypt
+except ImportError:
+    crypt = None
+
+from MoinMoin import log
+logging = log.getLogger(__name__)
 
 from MoinMoin.support.python_compatibility import hash_new, hmac_new
 
 from MoinMoin import config, caching, wikiutil, i18n, events
+from werkzeug.security import safe_str_cmp as safe_str_equal
 from MoinMoin.util import timefuncs, random_string
 from MoinMoin.wikiutil import url_quote_plus
 
+# for efficient lookup <attr> -> userid, we keep an index of this in the cache.
+# the attribute names in here should be uniquely identifying a user.
+CACHED_USER_ATTRS = ['name', 'email', 'jid', 'openids', ]
+
 
 def getUserList(request):
     """ Get a list of all (numerical) user IDs.
@@ -45,7 +58,10 @@ def getUserList(request):
     return userlist
 
 def get_by_filter(request, filter_func):
-    """ Searches for an user with a given filter function """
+    """ Searches for a user with a given filter function
+
+    Be careful: SLOW for big wikis, rather use _getUserIdByKey & related.
+    """
     for uid in getUserList(request):
         theuser = User(request, uid)
         if filter_func(theuser):
@@ -53,15 +69,17 @@ def get_by_filter(request, filter_func):
 
 def get_by_email_address(request, email_address):
     """ Searches for an user with a particular e-mail address and returns it. """
-    filter_func = lambda user: user.valid and user.email.lower() == email_address.lower()
-    return get_by_filter(request, filter_func)
+    uid = _getUserIdByKey(request, 'email', email_address, case=False)
+    if uid is not None:
+        return User(request, uid)
 
 def get_by_jabber_id(request, jabber_id):
     """ Searches for an user with a perticular jabber id and returns it. """
-    filter_func = lambda user: user.valid and user.jid.lower() == jabber_id.lower()
-    return get_by_filter(request, filter_func)
+    uid = _getUserIdByKey(request, 'jid', jabber_id, case=False)
+    if uid is not None:
+        return User(request, uid)
 
-def _getUserIdByKey(request, key, search):
+def _getUserIdByKey(request, key, search, case=True):
     """ Get the user ID for a specified key/value pair.
 
     This method must only be called for keys that are
@@ -69,43 +87,135 @@ def _getUserIdByKey(request, key, search):
 
     @param key: the key to look in
     @param search: the value to look for
+    @param case: do a case-sensitive lookup?
     @return the corresponding user ID or None
     """
-    if not search or not key:
+    if key not in CACHED_USER_ATTRS:
+        raise ValueError("unsupported key, must be in CACHED_USER_ATTRS")
+    if not search:
         return None
+    cfg_cache_attr = key + "2id"
+    if not case:
+        cfg_cache_attr += "_lower"
+        search = search.lower()
     cfg = request.cfg
-    cachekey = '%s2id' % key
     try:
-        _key2id = getattr(cfg.cache, cachekey)
+        attr2id = getattr(cfg.cache, cfg_cache_attr)
+        from_disk = False
     except AttributeError:
-        arena = 'user'
-        cache = caching.CacheEntry(request, arena, cachekey, scope='wiki', use_pickle=True)
-        try:
-            _key2id = cache.content()
-        except caching.CacheError:
-            _key2id = {}
-        setattr(cfg.cache, cachekey, _key2id)
-    uid = _key2id.get(search, None)
+        # no in-memory cache there - initialize it / load it from disk
+        loadLookupCaches(request)
+        attr2id = getattr(cfg.cache, cfg_cache_attr)
+        from_disk = True  # we just loaded the stuff from disk
+    uid = attr2id.get(search, None)
+    if uid is None and not from_disk:
+        # we do not have the entry we searched for.
+        # we didn't find it in some in-memory cache, try refreshing these from disk
+        loadLookupCaches(request)
+        attr2id = getattr(cfg.cache, cfg_cache_attr)
+        from_disk = True  # we just loaded the stuff from disk
+        uid = attr2id.get(search, None)
     if uid is None:
-        for userid in getUserList(request):
-            u = User(request, id=userid)
-            if hasattr(u, key):
-                value = getattr(u, key)
-                if isinstance(value, list):
-                    for val in value:
-                        _key2id[val] = userid
-                else:
-                    _key2id[value] = userid
-        arena = 'user'
-        cache = caching.CacheEntry(request, arena, cachekey, scope='wiki', use_pickle=True)
-        try:
-            cache.update(_key2id)
-        except caching.CacheError:
-            pass
-        uid = _key2id.get(search, None)
+        # we do not have the entry we searched for.
+        # we don't have it in the on-disk cache, cache MISS.
+        # could be because:
+        # a) ok: we have no such search value in the profiles
+        # b) fault: the cache is incoherent with the profiles
+        # c) fault: reading the cache from disk failed, due to an error
+        # d) ok: same as c), but just because no ondisk cache has been built yet
+        rebuildLookupCaches(request)  # XXX expensive
+        attr2id = getattr(cfg.cache, cfg_cache_attr)
+        uid = attr2id.get(search, None)
     return uid
 
 
+def setMemoryLookupCaches(request, cache):
+    """set the in-memory cache from the given cache contents
+
+    @param request: the request object
+    @param cache: either a dict of attrname -> attrcache to set the in-memory cache,
+                  or None to delete the in-memory cache.
+    """
+    for attrname in CACHED_USER_ATTRS:
+        if cache is None:
+            try:
+                delattr(request.cfg.cache, attrname + "2id")
+            except:
+                pass
+            try:
+                delattr(request.cfg.cache, attrname + "2id_lower")
+            except:
+                pass
+        else:
+            setattr(request.cfg.cache, attrname + "2id", cache[attrname])
+            setattr(request.cfg.cache, attrname + "2id_lower", cache[attrname + "_lower"])
+
+
+def loadLookupCaches(request):
+    """load lookup cache contents into memory: cfg.cache.XXX2id"""
+    scope, arena, cachekey = 'userdir', 'users', 'lookup'
+    diskcache = caching.CacheEntry(request, arena, cachekey, scope=scope, use_pickle=True)
+    try:
+        cache = diskcache.content()
+    except caching.CacheError:
+        cache = {}
+        for attrname in CACHED_USER_ATTRS:
+            cache[attrname] = {}
+    cache_with_lowercase = addLowerCaseKeys(cache)
+    setMemoryLookupCaches(request, cache_with_lowercase)
+
+
+def rebuildLookupCaches(request):
+    """complete attrs -> userid lookup cache rebuild"""
+    # as there may be thousands of users and reading all profiles is
+    # expensive, we just have 1 lookup cache for all interesting user attrs,
+    # so we only need to read all profiles ONCE to build the cache.
+    scope, arena, key = 'userdir', 'users', 'lookup'
+    diskcache = caching.CacheEntry(request, arena, key, scope=scope, use_pickle=True, do_locking=False)
+    diskcache.lock('w')
+
+    cache = {}
+    for attrname in CACHED_USER_ATTRS:
+        cache[attrname] = {}
+    for userid in getUserList(request):
+        u = User(request, id=userid)
+        if u.valid:
+            for attrname in CACHED_USER_ATTRS:
+                if hasattr(u, attrname):
+                    attr2id = cache[attrname]
+                    value = getattr(u, attrname)
+                    if isinstance(value, list):
+                        for val in value:
+                            attr2id[val] = userid
+                    else:
+                        attr2id[value] = userid
+
+    cache_with_lowercase = addLowerCaseKeys(cache)
+    setMemoryLookupCaches(request, cache_with_lowercase)
+    diskcache.update(cache)
+    diskcache.unlock()
+    return cache
+
+
+def clearLookupCaches(request):
+    """kill the userid lookup cache"""
+    # this triggers a rebuild of the cache.
+    setMemoryLookupCaches(request, None)
+    scope, arena, key = 'userdir', 'users', 'lookup'
+    caching.CacheEntry(request, arena, key, scope=scope).remove()
+
+
+def addLowerCaseKeys(cache):
+    """add lowercased lookup keys, so we can support case-insensitive lookup"""
+    c = deepcopy(cache)  # we do not want to modify cache itself
+    for attrname in CACHED_USER_ATTRS:
+        attr2id = c[attrname]
+        attr2id_lower = c[attrname + "_lower"] = {}
+        for key, value in attr2id.iteritems():
+            attr2id_lower[key.lower()] = value
+    return c
+
+
 def getUserId(request, searchName):
     """ Get the user ID for a specific user NAME.
 
@@ -126,6 +236,16 @@ def getUserIdByOpenId(request, openid):
     return _getUserIdByKey(request, 'openids', openid)
 
 
+def superusers(request):
+    """
+    yields superuser User objects
+    """
+    for name in request.cfg.superuser:
+        u = User(request, auth_username=name)
+        if u.isSuperUser():  # this checks for addtl. criteria
+            yield u
+
+
 def getUserIdentification(request, username=None):
     """ Return user name or IP or '<unknown>' indicator.
 
@@ -142,24 +262,75 @@ def getUserIdentification(request, username=None):
     return username or (request.cfg.show_hosts and request.remote_addr) or _("<unknown>")
 
 
-def encodePassword(pwd, salt=None):
-    """ Encode a cleartext password
+def encodePassword(cfg, pwd, salt=None, scheme=None):
+    """ Encode a cleartext password using the default algorithm.
 
+    @param cfg: the wiki config
     @param pwd: the cleartext password, (unicode)
-    @param salt: the salt for the password (string)
+    @param salt: the salt for the password (string) or None to generate a
+                 random salt.
+    @param scheme: scheme to use (by default will use cfg.password_scheme)
     @rtype: string
-    @return: the password in apache htpasswd compatible SHA-encoding,
-        or None
+    @return: the password hash in apache htpasswd compatible encoding,
     """
-    pwd = pwd.encode('utf-8')
+    if scheme is None:
+        scheme = cfg.password_scheme
+        configured_scheme = True
+    else:
+        configured_scheme = False
+    if scheme == '{PASSLIB}':
+        return '{PASSLIB}' + cfg.cache.pwd_context.encrypt(pwd, salt=salt)
+    elif scheme == '{SSHA}':
+        pwd = pwd.encode('utf-8')
+        if salt is None:
+            salt = random_string(20)
+        assert isinstance(salt, str)
+        hash = hash_new('sha1', pwd)
+        hash.update(salt)
+        return '{SSHA}' + base64.encodestring(hash.digest() + salt).rstrip()
+    else:
+        # should never happen as we check the value of cfg.password_scheme
+        raise NotImplementedError
 
-    if salt is None:
-        salt = random_string(20)
-    assert isinstance(salt, str)
-    hash = hash_new('sha1', pwd)
-    hash.update(salt)
 
-    return '{SSHA}' + base64.encodestring(hash.digest() + salt).rstrip()
+class Fault(Exception):
+    """something went wrong"""
+
+class NoSuchUser(Fault):
+    """raised if no such user exists"""
+
+class UserHasNoEMail(Fault):
+    """raised if user has no e-mail address in his profile"""
+
+class MailFailed(Fault):
+    """raised if e-mail sending failed"""
+
+
+def set_password(request, newpass, u=None, uid=None, uname=None,
+                 notify=False, skip_invalid=False, subject=None,
+                 text_intro=None, text_msg=None, text_data=None):
+    if uid:
+        u = User(request, uid)
+    elif uname:
+        u = User(request, auth_username=uname)
+    if u and u.exists():
+        if skip_invalid and u.enc_password == '':
+            return
+        if not newpass:
+            # set a invalid password hash
+            u.enc_password = ''
+        else:
+            u.enc_password = encodePassword(request.cfg, newpass)
+        u.save()
+        if not u.email:
+            raise UserHasNoEMail('User profile does not have an E-Mail address (name: %r id: %r)!' % (u.name, u.id))
+        if notify and not u.disabled:
+            mailok, msg = u.mailAccountData(subject=subject,
+                                            text_intro=text_intro, text_msg=text_msg, text_data=text_data)
+            if not mailok:
+                raise MailFailed(msg)
+    else:
+        raise NoSuchUser('User does not exist (name: %r id: %r)!' % (u.name, u.id))
 
 
 def normalizeName(name):
@@ -197,7 +368,7 @@ def isValidName(request, name):
     @param name: user name, unicode
     """
     normalized = normalizeName(name)
-    return (name == normalized) and not wikiutil.isGroupPage(request, name)
+    return (name == normalized) and not wikiutil.isGroupPage(name, request.cfg)
 
 
 def encodeList(items):
@@ -312,7 +483,7 @@ class User:
         self.recoverpass_key = ""
 
         if password:
-            self.enc_password = encodePassword(password)
+            self.enc_password = encodePassword(self._cfg, password)
 
         #self.edit_cols = 80
         self.tz_offset = int(float(self._cfg.tz_offset) * 3600)
@@ -332,7 +503,6 @@ class User:
 
         # attrs not saved to profile
         self._request = request
-        self._trail = []
 
         # we got an already authenticated username:
         check_password = None
@@ -351,7 +521,7 @@ class User:
         if not self.id:
             self.id = self.make_id()
             if password is not None:
-                self.enc_password = encodePassword(password)
+                self.enc_password = encodePassword(self._cfg, password)
 
         # "may" so we can say "if user.may.read(pagename):"
         if self._cfg.SecurityPolicy:
@@ -400,6 +570,10 @@ class User:
         """
         return os.path.exists(self.__filename())
 
+    def remove(self):
+        """ Remove user profile from disk """
+        os.remove(self.__filename())
+
     def load_from_id(self, password=None):
         """ Load user account data from disk.
 
@@ -478,9 +652,6 @@ class User:
         if -24 <= self.tz_offset and self.tz_offset <= 24:
             self.tz_offset = self.tz_offset * 3600
 
-        # clear trail
-        self._trail = []
-
         if not self.disabled:
             self.valid = 1
 
@@ -513,21 +684,75 @@ class User:
         if not password:
             return False, False
 
-        if epwd[:5] == '{SHA}':
-            enc = '{SHA}' + base64.encodestring(hash_new('sha1', password.encode('utf-8')).digest()).rstrip()
-            if epwd == enc:
-                data['enc_password'] = encodePassword(password) # upgrade to SSHA
-                return True, True
-            return False, False
+        password_correct = recompute_hash = False
+        wanted_scheme = self._cfg.password_scheme
 
-        if epwd[:6] == '{SSHA}':
-            data = base64.decodestring(epwd[6:])
-            salt = data[20:]
-            hash = hash_new('sha1', password.encode('utf-8'))
-            hash.update(salt)
-            return hash.digest() == data[:20], False
+        # Check password and upgrade weak hashes to strong default algorithm:
+        for scheme in config.password_schemes_supported:
+            if epwd.startswith(scheme):
+                is_passlib = False
+                d = epwd[len(scheme):]
 
-        # No encoded password match, this must be wrong password
+                if scheme == '{PASSLIB}':
+                    # a password hash to be checked by passlib library code
+                    if not self._cfg.passlib_support:
+                        logging.error('in user profile %r, password hash with {PASSLIB} scheme encountered, but passlib_support is False' % (self.id, ))
+                    else:
+                        pwd_context = self._cfg.cache.pwd_context
+                        try:
+                            password_correct = pwd_context.verify(password, d)
+                        except ValueError, err:
+                            # can happen for unknown scheme
+                            logging.error('in user profile %r, verifying the passlib pw hash crashed [%s]' % (self.id, str(err)))
+                        if password_correct:
+                            # check if we need to recompute the hash. this is needed if either the
+                            # passlib hash scheme / hash params changed or if we shall change to a
+                            # builtin hash scheme (not recommended):
+                            recompute_hash = pwd_context.hash_needs_update(d) or wanted_scheme != '{PASSLIB}'
+
+                else:
+                    # a password hash to be checked by legacy, builtin code
+                    if scheme == '{SSHA}':
+                        d = base64.decodestring(d)
+                        salt = d[20:]
+                        hash = hash_new('sha1', password.encode('utf-8'))
+                        hash.update(salt)
+                        enc = base64.encodestring(hash.digest() + salt).rstrip()
+
+                    elif scheme == '{SHA}':
+                        enc = base64.encodestring(
+                            hash_new('sha1', password.encode('utf-8')).digest()).rstrip()
+
+                    elif scheme == '{APR1}':
+                        # d is of the form "$apr1$<salt>$<hash>"
+                        salt = d.split('$')[2]
+                        enc = md5crypt.apache_md5_crypt(password.encode('utf-8'),
+                                                        salt.encode('ascii'))
+                    elif scheme == '{MD5}':
+                        # d is of the form "$1$<salt>$<hash>"
+                        salt = d.split('$')[2]
+                        enc = md5crypt.unix_md5_crypt(password.encode('utf-8'),
+                                                      salt.encode('ascii'))
+                    elif scheme == '{DES}':
+                        if crypt is None:
+                            return False, False
+                        # d is 2 characters salt + 11 characters hash
+                        salt = d[:2]
+                        enc = crypt.crypt(password.encode('utf-8'), salt.encode('ascii'))
+
+                    else:
+                        logging.error('in user profile %r, password hash with unknown scheme encountered: %r' % (self.id, scheme))
+                        raise NotImplementedError
+
+                    if safe_str_equal(epwd, scheme + enc):
+                        password_correct = True
+                        recompute_hash = scheme != wanted_scheme
+
+                if recompute_hash:
+                    data['enc_password'] = encodePassword(self._cfg, password)
+                return password_correct, recompute_hash
+
+        # unsupported algorithm
         return False, False
 
     def persistent_items(self):
@@ -573,28 +798,19 @@ class User:
             data.write(line + '\n')
         data.close()
 
-        arena = 'user'
-        key = 'name2id'
-        caching.CacheEntry(self._request, arena, key, scope='wiki').remove()
-        try:
-            del self._request.cfg.cache.name2id
-        except:
-            pass
-        key = 'openid2id'
-        caching.CacheEntry(self._request, arena, key, scope='wiki').remove()
-        try:
-            del self._request.cfg.cache.openid2id
-        except:
-            pass
-
         if not self.disabled:
             self.valid = 1
 
+        self.updateLookupCaches()
+
         if not self._stored:
             self._stored = True
             event = events.UserCreatedEvent(self._request, self)
             events.send_event(event)
 
+        # update page subscriber's cache after saving user preferences
+        self.updatePageSubCache()
+
     # -----------------------------------------------------------------
     # Time and date formatting
 
@@ -691,9 +907,6 @@ class User:
         The subscription list may contain page names or interwiki page
         names. e.g 'Page Name' or 'WikiName:Page_Name'
 
-        TODO: check if it's fast enough when getting called for many
-              users from page.getSubscribersList()
-
         @param pagelist: list of pages to check for subscription
         @rtype: bool
         @return: if user is subscribed any page in pagelist
@@ -782,6 +995,70 @@ class User:
             self.save()
         return not self.isSubscribedTo([pagename])
 
+    def updatePageSubCache(self):
+        """ When a user profile is saved, we update the page subscriber's cache """
+
+        scope, arena, key = 'userdir', 'users', 'pagesubscriptions'
+
+        cache = caching.CacheEntry(self._request, arena=arena, key=key, scope=scope, use_pickle=True, do_locking=False)
+        if not cache.exists():
+            return  # if no cache file exists, just don't do anything
+
+        cache.lock('w')
+        page_sub = cache.content()
+
+        # we only store entries for valid users with some page subscriptions
+        if self.valid and self.subscribed_pages:
+            page_sub[self.id] = {
+                'name': self.name,
+                'email': self.email,
+                'subscribed_pages': self.subscribed_pages,
+            }
+        elif page_sub.get(self.id):
+            del page_sub[self.id]
+
+        cache.update(page_sub)
+        cache.unlock()
+
+    def updateLookupCaches(self):
+        """ When a user profile is saved, we update the userid lookup caches """
+
+        scope, arena, key = 'userdir', 'users', 'lookup'
+
+        diskcache = caching.CacheEntry(self._request, arena=arena, key=key, scope=scope, use_pickle=True, do_locking=False)
+        if not diskcache.exists():
+            return  # if no cache file exists, just don't do anything
+
+        diskcache.lock('w')
+        cache = diskcache.content()
+        userid = self.id
+
+        # first remove all old entries mapping to this userid:
+        for attrname in CACHED_USER_ATTRS:
+            attr2id = cache[attrname]
+            for key, value in attr2id.items():
+                if value == userid:
+                    del attr2id[key]
+
+        # then, if user is valid, update with the current attr values:
+        if self.valid:
+            for attrname in CACHED_USER_ATTRS:
+                if hasattr(self, attrname):
+                    value = getattr(self, attrname)
+                    if value:
+                        # we do not store empty values, likely not unique
+                        attr2id = cache[attrname]
+                        if isinstance(value, list):
+                            for val in value:
+                                attr2id[val] = userid
+                        else:
+                            attr2id[value] = userid
+
+        cache_with_lowercase = addLowerCaseKeys(cache)
+        setMemoryLookupCaches(self._request, cache_with_lowercase)
+        diskcache.update(cache)
+        diskcache.unlock()
+
     # -----------------------------------------------------------------
     # Quicklinks
 
@@ -879,7 +1156,7 @@ class User:
     # Trail
 
     def _wantTrail(self):
-        return (not self.valid and self._request.session.is_stored  # anon session
+        return (not self.valid and self._request.cfg.cookie_lifetime[0]  # anon sessions enabled
                 or self.valid and (self.show_page_trail or self.remember_last_visit))  # logged-in session
 
     def addTrail(self, page):
@@ -888,9 +1165,6 @@ class User:
         @param page: the page (object) to add to the trail
         """
         if self._wantTrail():
-            # load trail if not known
-            self.getTrail()
-
             pagename = page.page_name
             # Add only existing pages that the user may read
             if not (page.exists() and self._request.user.may.read(pagename)):
@@ -900,20 +1174,22 @@ class User:
             if self._cfg.interwikiname:
                 pagename = self._interWikiName(pagename)
 
+            trail = self._request.session.get('trail', [])
+            trail_current = trail[:]
+
             # Don't append tail to trail ;)
-            if self._trail and self._trail[-1] == pagename:
+            if trail and trail[-1] == pagename:
                 return
 
             # Append new page, limiting the length
-            self._trail = [p for p in self._trail if p != pagename]
-            self._trail = self._trail[-(self._cfg.trail_size-1):]
-            self._trail.append(pagename)
-            self.saveTrail()
-
-    def saveTrail(self):
-        """ Save trail into session """
-        if not self._request.session.is_new:
-            self._request.session['trail'] = self._trail
+            trail = [p for p in trail if p != pagename]
+            pagename_stripped = pagename.strip()
+            if pagename_stripped:
+                trail.append(pagename_stripped)
+            trail = trail[-self._cfg.trail_size:]
+            if trail != trail_current:
+                # we only modify the session if we have something different:
+                self._request.session['trail'] = trail
 
     def getTrail(self):
         """ Return list of recently visited pages.
@@ -921,13 +1197,11 @@ class User:
         @rtype: list
         @return: pages in trail
         """
-        if not self._trail and self._wantTrail():
+        if self._wantTrail():
             trail = self._request.session.get('trail', [])
-            trail = [t.strip() for t in trail]
-            trail = [t for t in trail if t]
-            self._trail = trail[-self._cfg.trail_size:]
-
-        return self._trail
+        else:
+            trail = []
+        return trail
 
     # -----------------------------------------------------------------
     # Other
@@ -938,13 +1212,15 @@ class User:
 
     def isSuperUser(self):
         """ Check if this user is superuser """
+        if not self.valid:
+            return False
         request = self._request
-        if request.cfg.DesktopEdition and request.remote_addr == '127.0.0.1' and request.user and request.user.valid:
+        if request.cfg.DesktopEdition and request.remote_addr == '127.0.0.1':
             # the DesktopEdition gives any local user superuser powers
             return True
         superusers = request.cfg.superuser
         assert isinstance(superusers, (list, tuple))
-        return self.valid and self.name and self.name in superusers
+        return self.name and self.name in superusers
 
     def host(self):
         """ Return user host """
@@ -997,53 +1273,62 @@ class User:
             stamp = int(parts[0])
         except ValueError:
             return False
-        # only allow it to be valid for twelve hours
-        if stamp + 12*60*60 < time.time():
+        lifetime = self._request.cfg.recovery_token_lifetime * 3600
+        if time.time() > stamp + lifetime:
             return False
         # check hmac
         # key must be of type string
         h = hmac_new(str(self.recoverpass_key), str(stamp)).hexdigest()
-        if h != parts[1]:
+        if not safe_str_equal(h, parts[1]):
             return False
         self.recoverpass_key = ""
-        self.enc_password = encodePassword(newpass)
+        self.enc_password = encodePassword(self._cfg, newpass)
         self.save()
         return True
 
-    def mailAccountData(self, cleartext_passwd=None):
+    def mailAccountData(self, cleartext_passwd=None,
+                        subject=None,
+                        text_intro=None, text_msg=None, text_data=None):
         """ Mail a user who forgot his password a message enabling
             him to login again.
         """
         from MoinMoin.mail import sendmail
-        from MoinMoin.wikiutil import getLocalizedPage
         _ = self._request.getText
 
+        if not self.email:
+            return False, "user has no E-Mail address in his profile."
+
         tok = self.generate_recovery_token()
 
-        text = '\n' + _("""\
-Login Name: %s
-
-Password recovery token: %s
-
-Password reset URL: %s/?action=recoverpass&name=%s&token=%s
-""") % (
-                        self.name,
-                        tok,
-                        self._request.getBaseURL(),
-                        url_quote_plus(self.name),
-                        tok, )
-
-        text = _("""\
+        if subject is None:
+            subject = _('[%(sitename)s] Your wiki account data')
+        subject = subject % dict(sitename=self._cfg.sitename or "Wiki")
+        if text_intro is None:
+            text_intro = ''
+        if text_msg is None:
+            text_msg = _("""\
 Somebody has requested to email you a password recovery token.
 
 If you lost your password, please go to the password reset URL below or
 go to the password recovery page again and enter your username and the
 recovery token.
-""") + text
+""")
+        if text_data is None:
+            text_data = _("""\
+Login Name: %s
 
+Password recovery token: %s
+
+Password reset URL: %s?action=recoverpass&name=%s&token=%s
+""")
+        # note: text_intro is for custom stuff, we do not have i18n for it anyway
+        text = text_intro + '\n' + _(text_msg) + '\n' + _(text_data) % (
+                        self.name,
+                        tok,
+                        self._request.url, # use full url, including current page
+                        url_quote_plus(self.name),
+                        tok, )
 
-        subject = _('[%(sitename)s] Your wiki account data',
-                ) % {'sitename': self._cfg.sitename or "Wiki"}
         mailok, msg = sendmail.sendmail(self._request, [self.email], subject,
                                     text, mail_from=self._cfg.mail_from)
         return mailok, msg
