@@ -121,7 +121,7 @@ def url_unquote(s, want_unicode=None):
 def parseQueryString(qstr, want_unicode=None):
     """ see werkzeug.url_decode
 
-        Please note: this returns a MultiDict, you might need to use dict() on
+        Please note: this returns a MultiDict, you might need to use .to_dict() on
                      the result if your code expects a "normal" dict.
     """
     try:
