@@ -134,7 +134,7 @@ class XmlRpcBase:
                 # xmlrpc, you have to use multicall and getAuthToken / applyAuthToken
                 request.user = user.User(request, auth_method='xmlrpc:invalid')
 
-                data = request.in_data
+                data = request.read()
 
                 try:
                     params, method = xmlrpclib.loads(data)
@@ -447,6 +447,36 @@ class XmlRpcBase:
             'version': version,
             }
 
+    def xmlrpc_getProcessingInstruction(self, pagename, pi):
+        """
+        Invoke getProcessingInstructionVersion with rev=None
+        """
+        return self.xmlrpc_getProcessingInstructionVersion(pagename, rev=None, pi=pi)
+
+    def xmlrpc_getProcessingInstructionVersion(self, pagename, rev, pi):
+        """
+        Returns value of Processing Instruction
+        @param pagename: pagename (utf-8)
+        @param rev: revision number (int)
+        @param pi: PI key
+        """
+        # User may read page?
+        if not self.request.user.may.read(pagename):
+            return self.notAllowedFault()
+        if rev is not None:
+            page = Page(self.request, pagename, rev=rev)
+        else:
+            page = Page(self.request, pagename)
+        if not page.exists():
+            return xmlrpclib.Fault("NOT_EXIST", "Page does not exist.")
+        if pi is None:
+            return xmlrpclib.Fault("NOT_EXIST", "Processing Instruction not given.")
+        try:
+            instruction = page.pi[pi]
+        except KeyError:
+            return xmlrpclib.Fault("NOT_EXIST", "Processing Instruction does not exist.")
+        return self._outstr(instruction)
+
     def xmlrpc_getPage(self, pagename):
         """
         Invoke xmlrpc_getPageVersion with rev=None
@@ -567,7 +597,7 @@ class XmlRpcBase:
         if not self.request.user.may.write(pagename):
             return xmlrpclib.Fault(1, "You are not allowed to edit this page")
 
-        page = PageEditor(self.request, pagename)
+        page = PageEditor(self.request, pagename, do_editor_backup=0)
         try:
             if self.version == 2:
                 newtext = self._instr(pagetext)
@@ -601,7 +631,7 @@ class XmlRpcBase:
         # check ACLs
         if not (self.request.user.may.delete(pagename) and self.request.user.may.write(newpagename)):
             return xmlrpclib.Fault(1, "You are not allowed to rename this page")
-        editor = PageEditor(self.request, pagename)
+        editor = PageEditor(self.request, pagename, do_editor_backup=0)
 
         try:
             editor.renamePage(newpagename)
@@ -629,7 +659,7 @@ class XmlRpcBase:
             return xmlrpclib.Fault(1, "You are not allowed to edit this page")
 
         rev = int(self._instr(revision))
-        editor = PageEditor(self.request, pagename)
+        editor = PageEditor(self.request, pagename, do_editor_backup=0)
 
         try:
             editor.revertPage(rev)
